/*******************************************************************************
 * Copyright (c) 2014-2015 Marian Schedenig
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marian Schedenig - initial API and implementation
 *******************************************************************************/

package name.schedenig.eclipse.clippets.java.utils;

import name.schedenig.eclipse.clippets.java.Activator;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

public class JavaUtils
{
	public static IType getJavaType(ITextEditor editor)
	{
		return (IType) getJavaElement(editor, IJavaElement.TYPE);
	}
	
	public static IMethod getJavaMethod(ITextEditor editor)
	{
		return (IMethod) getJavaElement(editor, IJavaElement.METHOD);
	}
	
	public static IPackageFragment getJavaPackage(ITextEditor editor)
	{
		IPackageFragment pkg = (IPackageFragment) getJavaElement(editor, IJavaElement.PACKAGE_FRAGMENT);
		
		if(pkg == null)
		{
			ICompilationUnit unit = getJavaCompilationUnit(editor);
			
			if(unit != null)
			{
				pkg = (IPackageFragment) unit.getAncestor(IJavaElement.PACKAGE_FRAGMENT);
			}
		}
		
		return pkg;
	}
	
	public static ICompilationUnit getJavaCompilationUnit(ITextEditor editor)
	{
		IEditorInput editorInput = editor.getEditorInput();
		IJavaElement elem = JavaUI.getEditorInputJavaElement(editorInput);
		
		if(elem instanceof ICompilationUnit)
		{
			return (ICompilationUnit) elem;
		}
		else
		{
			return null;
		}
	}
	
	public static IJavaElement getJavaElement(ITextEditor editor, int type)
	{
		ICompilationUnit unit = getJavaCompilationUnit(editor);
		
		if(unit == null)
		{
			return null;
		}
		
		ITextSelection selection = (ITextSelection) editor.getSelectionProvider().getSelection();
		IJavaElement selected;
		
		try
		{
			selected = unit.getElementAt(selection.getOffset());
		}
		catch(JavaModelException ex)
		{
			Activator.getDefault().log(IStatus.ERROR, ex);
			return null;
		}

		if(selected == null)
		{
			return null;
		}

		return selected.getAncestor(type);
	}
}
